//   C4D3 is free software: you can redistribute it and/or modify it under
// the terms of the GNU Affero General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option) any
// later version.
import { ControlInfo } from "./ControlInfo.js";
import { ControlEvent } from "./ControlEvent.js";
import { SharedFlag } from "./SharedFlag.js";
import { LiveProperty } from "./LiveProperty.js";
import { printClass } from "./printClass.js";
//type typeOfValue = string | number | boolean | undefined | null;
export class ControlProxy {
    //**********************************************************************
    // Constructors and Finalizer
    //**********************************************************************
    constructor(control) {
        this.tags = []; //prototype names
        this.map = new Map(); // Tag->LiveProperty
        // Working (transient) variables
        //private const event: StateEvent;
        this.interacting = null;
        this.lastInteractionTime = null;
        this.lastUpdateTime = null;
        // Event Multicasters
        this.controlListeners = [];
        this.stateListeners = [];
        //if(printClass.printStatus) console.log("Hi I am in control proxy");
        //if(printClass.printStatus) console.log(control);
        this.control = control;
        this.info = new ControlInfo(this);
        this.tags = [];
        this.map = new Map();
        //event = new StateEvent(this);
        this.metaFocusingFlag = new SharedFlag();
        this.metaEditingFlag = new SharedFlag();
        //this.controlListeners = [4];
        //this.stateListeners = [4];
        if (printClass.printStatus)
            console.log("Second: ControlProxy Object Created");
        //if(printClass.printStatus) console.log(this);
    }
    //**********************************************************************
    // Getters and Setters
    //**********************************************************************
    getControl() {
        return this.control;
    }
    //**********************************************************************
    // Public Methods (Control)
    //**********************************************************************
    describe() {
        return this.info;
    }
    //**********************************************************************
    // Public Methods (Properties)
    //**********************************************************************
    /**
     * Returns a vector of all tags, in the order added.
     */
    getTags() {
        var s = [];
        s = this.tags;
        return s;
    }
    /**
     * Returns the number of tags.
     */
    getTagCount() {
        return this.tags.length;
    }
    /**
     * Returns the tag at the specified index.
     */
    getTagAt(index) {
        return this.tags[index];
    }
    /**
     * Returns where a tag is in the vector of all tags, or -1 if absent.
     */
    getIndexOfTag(tag) {
        return this.tags.indexOf(tag);
    }
    /**
     * Returns the LiveProperty associated with the specified tag, or null if
     * none exists.
     *
     * @param		tag		the name of the LiveProperty to retrieve
     */
    getLiveProperty(tag) {
        if (printClass.printStatus)
            console.log(ControlProxy.map.get(tag));
        if (printClass.printStatus)
            console.log("getLiveProperty method of ControlProxy");
        if (printClass.printStatus)
            console.log(this.map.get(tag));
        return this.map.get(tag);
    }
    /**
     * Returns a vector of all LiveProperties, in the order added.
     */
    getLiveProperties() {
        var properties = [];
        var i = 0;
        for (i = 0; i < properties.length; i++)
            properties[i] = this.map.get(this.tags[i]);
        return properties;
    }
    /**
     * Returns the property at the specified index.
     */
    getPropertyAt(index) {
        return this.map.get(this.tags[index]);
    }
    /**
     * Convenience method to register a listener with all current properties.
     * This does NOT automatically handle future addition and removal of
     * properties to the control. (Use ControlListener to detect such changes.)
     */
    addLivePropertyListener(l) {
        var n = this.tags.length;
        var i;
        for (i = 0; i < n; i++) {
            var tag = this.tags[i];
            var property = this.map.get(tag);
            property.addLivePropertyListener(l);
        }
    }
    /**
     * Convenience method to deregister a listener with all current properties.
     * This does NOT automatically handle future addition and removal of
     * properties to the control. (Use ControlListener to detect such changes.)
     */
    removeLivePropertyListener(l) {
        var n = this.tags.length;
        var i;
        for (i = 0; i < n; i++) {
            var tag = this.tags[i];
            var property = this.map.get(tag);
            property.removeLivePropertyListener(l);
        }
    }
    add(tag, prototype, interactive) {
        if (interactive) {
            if (this.map.has(tag))
                throw new Error("Duplicate tag: " + tag);
            var property = new LiveProperty(this.control, tag, prototype, interactive);
            this.tags.push(tag);
            //if(printClass.printStatus) console.log("I am in here ");
            this.map.set(tag, property);
            if (printClass.printStatus)
                console.log(this.map.get(tag)); //sending undefined value
            property.addLivePropertyListener(this.control);
            //if(printClass.printStatus) console.log("Hi")
            this.firePropertyAdded(new ControlEvent(this.control, property));
            return property;
        }
        else {
            if (this.map.has(tag))
                throw new Error("Duplicate tag: " + tag);
            var property = new LiveProperty(this.control, tag, prototype, false);
            this.tags.push(tag);
            this.map.set(tag, property);
            property.addLivePropertyListener(this.control);
            this.firePropertyAdded(new ControlEvent(this.control, property));
            return property;
        }
    }
    remove(tag) {
        var property = this.map.get(tag);
        if (property == null)
            throw new Error("Unknown tag: " + tag);
        property.removeLivePropertyListener(this.control);
        this.map.delete(tag);
        this.tags.splice(this.tags.indexOf(tag), 1);
        this.firePropertyRemoved(new ControlEvent(this.control, property));
        property.dispose();
    }
    removeAll() {
        while (this.tags.length > 0)
            this.remove(this.tags[this.tags.length - 1]);
    }
    //**********************************************************************
    // Public Methods (Properties, Variable Access)
    //********************************************************************* 
    getVariable(tag) {
        return this.getLiveProperty(tag).getVariable();
    }
    setVariable(tag, variable) {
        if (printClass.printStatus)
            console.log("setVariable Method of ControlProxy");
        if (printClass.printStatus)
            console.log(this.getLiveProperty(tag));
        if (printClass.printStatus)
            console.log(variable);
        this.getLiveProperty(tag).setVariable(variable);
        //if(printClass.printStatus) console.log(this.getLiveProperty(tag));
    }
    getName(tag) {
        return this.getLiveProperty(tag).getName();
    }
    setName(tag, name) {
        this.getLiveProperty(tag).setName(name);
    }
    getValue(tag) {
        var value = this.getLiveProperty(tag).getValue();
        return value;
    }
    setValue(tag, value) {
        console.log("is my server working");
        this.getLiveProperty(tag).setValue(value);
    }
    //**********************************************************************
    // Public Methods (Properties, Locking)
    //**********************************************************************
    isLocked() {
        var n = this.tags.length;
        var i;
        for (i = 0; i < n; i++) {
            var tag = this.tags[i].toString();
            var property = this.map.get(tag);
            if (property.getInteractive() && !property.isLocked())
                return false;
        }
        return true;
    }
    //**********************************************************************
    // Public Methods (Events)
    //**********************************************************************
    /**
     * Registers the specified ControlListener to receive ControlEvents
     * whenever a LiveProperty is added to or removed from this this.
     */
    addControlListener(l) {
        this.controlListeners.push(l);
    }
    /**
     * Unregisters the specified ControlListener.
     */
    removeControlListener(l) {
        this.controlListeners.splice(this.controlListeners.indexOf(l.toString()), 1);
    }
    //**********************************************************************
    // Override Methods (Disposable)
    //**********************************************************************
    dispose() {
        while (this.tags.length > 0) {
            var tag = this.tags[this.tags.length - 1].toString();
            if (!tag.endsWith(".Mutator"))
                this.remove(tag);
        }
        this.controlListeners.splice(0, this.controlListeners.length);
        this.stateListeners.splice(0, this.stateListeners.length);
    }
    //**********************************************************************
    // Private Methods (Events)
    //**********************************************************************
    firePropertyAdded(e) {
        //if(printClass.printStatus) console.log("I am in firePropertyAdded method of this");
        var n = this.controlListeners.length;
        var i;
        for (i = 0; i < n; i++)
            (this.controlListeners[i]).propertyAdded(e);
    }
    firePropertyRemoved(e) {
        var n = this.controlListeners.length;
        var i;
        for (i = 0; i < n; i++)
            (this.controlListeners[i]).propertyRemoved(e);
    }
    fireStateChanged() {
        var n = this.stateListeners.length;
        var i;
        for (i = 0; i < n; i++)
            (this.stateListeners[i]).stateChanged(event);
    }
}
//******************************************************************************
//# sourceMappingURL=ControlProxy.js.map